<?php
/**
 * YJShopping MediaWiki extension  version 1.1.2
 *	for details please see: https://kimagurenote.net/kn/YJShopping
 *
 * Copyright (c) 2019 Kimagurenote https://kimagurenote.net/
 * License: Revised BSD license http://opensource.org/licenses/BSD-3-Clause
 *
 * Function:
 *	Mediawiki extension that make item links for several shopping site.
 *
 * Usage:
 *	<yjshopping>free keyword</shopping> on your wiki page.
 *	for details please see: https://kimagurenote.net/kn/YJShopping
 *
 * History:
 * 2019.06.02 Version 1.1.2
 *	Sofmap.com, Sofmapリコレ（中古） (LinkShare) に対応
 *	楽天市場 (LinkShare) に対応
 * 2019.05.30 Version 1.1.1
 *	関数出力が後々（MediaWiki側で）勝手にescapeされることがある問題に仮対策（未解決、動作はしているようだが…）
 *	Amazon.co.jp に表記変更、Amazon.co.jp の ValueCommerce MyLink に対応
 *	内部変数 $wgYJShoppingImgStyle を廃止、float:left の直接指定を削除（要CSS）
 *	定数YJSHOPPING_DEBUGを追加、debug code を存置するとともに通常は通さないように
 *	Web Services by Yahoo! JAPAN クレジットを追加 https://developer.yahoo.co.jp/attribution/
 *	バージョン番号表記をMediaWiki似に変更
 * 2019.05.26 Version 1.10
 *	ValueCommerce系の処理を統合して関数に分離、Amazonと楽天の処理も関数に分離
 *	'yj' を 'yahoo' に変更（当面の間、互換性処理を追加）
 *	帯域節約のため、先頭の1件のみ取得するようにした
 *	与えられた引数を mb_convert_encoding() に通すようにした（EUC等を使っているWiki対策）
 * 2019.05.24 Version 1.00
 *	1st test
 *
 * @file
 * @ingroup Extensions
 * @author Kimagurenote
 * @copyright © 2019 Kimagurenote
 * @license The BSD 3-Clause License
 */

if( !defined( 'MEDIAWIKI' ) ) {
	echo( "This file is an extension to the MediaWiki software and cannot be used standalone.\n" );
	die( 1 );
}

$wgExtensionCredits['parserhook'][] = array(
	'path'           => __FILE__,
	'name'           => 'YJShopping',
	'author'         => array(
		'[https://kimagurenote.net/ Kimagurenote]'
	),
	'url'            => 'https://kimagurenote.net/kn/YJShopping',
	'version'        => '1.1.2',
	'license-name'   => 'BSD',
	'descriptionmsg' => 'yjshopping-desc'
);

/* Internationalization */
$wgMessagesDirs['YJShopping'] = __DIR__ . '/i18n';

/* default values */
define( 'YJSHOPPING_DEBUG', false );
define( 'YJSHOPPING_CREDIT_URL', 'https://s.yimg.jp/images/yjdn/common/yjdn_attbtn1_125_17.gif' );
define( 'YJSHOPPING_CREDIT_SIZE_X', 125 );
define( 'YJSHOPPING_CREDIT_SIZE_Y', 17 );
define( 'YJSHOPPING_CREDIT_ALT', 'Web Services by Yahoo! JAPAN' );
define( 'YJSHOPPING_CREDIT_TARGET', 'https://developer.yahoo.co.jp/about' );
define( 'YJSHOPPING_HOOKID', 'yjshopping' );
define( 'YJSHOPPING_CLASS_DIV', 'yjshopping' );

/* query type identifier */
define( 'YJSHOPPING_QUERYTYPE_ISBN', 'isbn' );
define( 'YJSHOPPING_QUERYTYPE_JAN', 'jan' );

/* operation parameter */
define( 'YJSHOPPING_NAMEMAXLEN', -1 );
define( 'YJSHOPPING_STOREMAXLEN', -1 );

/* Yahoo!JAPAN Shopping API URL (json)
 * https://developer.yahoo.co.jp/webapi/shopping/shopping/v1/itemsearch.html
 */
$wgYJShoppingAPIURL = 'https://shopping.yahooapis.jp/ShoppingWebService/V1/json/itemSearch';

/* tag classes */
$wgYJShoppingClass['div'] = '';
$wgYJShoppingClass['img'] = 'yjshoppingimg';
$wgYJShoppingClass['name'] = 'yjshoppingname';
$wgYJShoppingClass['store'] = 'yjshoppingstore';
$wgYJShoppingClass['plus'] = 'yjshoppingplus';
$wgYJShoppingClass['yahoo'] = 'shoplinkyahoo';
$wgYJShoppingClass['amazon'] = 'shoplinkamazon';
$wgYJShoppingClass['rakuten'] = 'shoplinkrakuten';
$wgYJShoppingClass['sofmap'] = 'shoplinksofmap';
$wgYJShoppingClass['sofmapused'] = 'shoplinksofmapused';
$wgYJShoppingClass['honto'] = 'shoplinkhonto';
$wgYJShoppingClass['7net'] = 'shoplink7net';
$wgYJShoppingClass['wowma'] = 'shoplinkwowma';
$wgYJShoppingClass['kakakucom'] = 'shoplinkkakakucom';

/* accepted parameters for pass to API */
$wgYJShoppingAcceptArgs = array( 'image_size', 'category_id', 'product_id', 'person_id', 'brand_id', 'store_id', 'sort', 'condition' );


/* MediaWiki set hook */
$wgHooks['ParserFirstCallInit'][] = 'wfYJShoppingSetHook';

function wfYJShoppingSetHook( $parser ) {
	$parser->setHook( YJSHOPPING_HOOKID, 'wfRenderYJShopping' );
	return true;
}

/* Main process - The callback function for embed shopping link
 *
 */
function wfRenderYJShopping( $value, $argv, $parser ) {
	global $wgYJShoppingAPIURL, $wgYJShoppingAppID, $wgYJShoppingVCID, $wgYJShoppingAcceptArgs, $wgYJShoppingClass, $wgYJShoppingMaxLen, $wgYJShoppingPlus;

	/* for debug */	YJSHOPPING_DEBUG && ( $parser->disableCache() );

	/* search keyword */
	$value = mb_convert_encoding(trim($value), 'UTF8', 'ASCII,JIS,UTF-8,EUC-JP,SJIS-WIN');

	/* require apiurl, appid and keyword(value) */
	if( empty($wgYJShoppingAPIURL) || empty($wgYJShoppingAppID) || empty($value) ) {
		/* for debug */	YJSHOPPING_DEBUG && ( wfDebugLog( YJSHOPPING_HOOKID, "wfRenderYJShopping(): required parameter(s) empty: $wgYJShoppingAPIURL, $wgYJShoppingAppID, $value.\n" ) );
		return '';
	}

	/* backward compatibility */
	if( !isset($wgYJShoppingPlus['yahoo']) && !empty($wgYJShoppingPlus['yj']) ) {
		$wgYJShoppingPlus['yahoo'] = $wgYJShoppingPlus['yj'];
		unset($wgYJShoppingPlus['yj']);
	}

	/* generate API URL */
	$apiurl = $wgYJShoppingAPIURL . '?appid=' . $wgYJShoppingAppID;

	/* ValueCommerce affiliate ID (option)
	 * https://developer.yahoo.co.jp/appendix/shopping/affiliate.html
	 */
	if( !empty($wgYJShoppingVCID) ) {
		$apiurl .= '&affiliate_type=vc&affiliate_id=' . urlencode($wgYJShoppingVCID);
	} elseif ( !empty($wgYJShoppingPlus['yahoo']['sid']) && !empty($wgYJShoppingPlus['yahoo']['pid']) ) {
		$v = 'https://ck.jp.ap.valuecommerce.com/servlet/referral?sid=' . $wgYJShoppingPlus['yahoo']['sid'] . '&pid=' . $wgYJShoppingPlus['yahoo']['pid'] . '&vc_url=';
		$apiurl .= '&affiliate_type=vc&affiliate_id=' . urlencode($v);
	}

	/* select query type (default: keyword search) */
	$querytype = @$argv['querytype'];
	if ( strcasecmp($querytype, YJSHOPPING_QUERYTYPE_ISBN) == 0 ) {
		$querytype = YJSHOPPING_QUERYTYPE_ISBN;
		$value = str_replace('-', '', $value);
		$apiurl .= '&isbn=';
	} elseif ( strcasecmp($querytype, YJSHOPPING_QUERYTYPE_JAN) == 0 ) {
		$querytype = YJSHOPPING_QUERYTYPE_JAN;
		$apiurl .= '&jan=';
	} else {
		$querytype = '';
		$apiurl .= '&query=';
	}
	$apiurl .= urlencode($value);

	/* set result count */
	$apiurl .= '&hits=1';

	/* more parameters if accepted (option) */
	if( !empty($args = @$argv) ) {
		foreach( $wgYJShoppingAcceptArgs as $v ) {
			if( !empty($args[$v]) ) {
				$apiurl .= '&' . $v . '=' . urlencode($args[$v]);
				unset($args[$v]);
			}
		}
	}

	/* call API */
	if( ($retapi = file_get_contents($apiurl)) === false ) {
		/* for debug */	YJSHOPPING_DEBUG && ( wfDebugLog( YJSHOPPING_HOOKID, "wfRenderYJShopping(): can't call api $apiurl.\n" ) );
		return '';
	}

	/* for debug */	YJSHOPPING_DEBUG && ( wfDebugLog( YJSHOPPING_HOOKID, "wfRenderYJShopping(): API call success.\n$apiurl\n" ) );

	/* decode JSON */
	if( is_null( $resapi = json_decode( mb_convert_encoding( $retapi, 'UTF8', 'ASCII,JIS,UTF-8,EUC-JP,SJIS-WIN'), true ) ) ) {
		/* for debug */	YJSHOPPING_DEBUG && ( wfDebugLog( YJSHOPPING_HOOKID, "wfRenderYJShopping(): can't decode json.\n" . $resapi . "\n" ) );
		return '';
	}

	/* no result */
	if( $resapi['ResultSet']['totalResultsReturned'] < 1 ) {
		/* for debug */	YJSHOPPING_DEBUG && ( wfDebugLog( YJSHOPPING_HOOKID, "wfRenderYJShopping(): no result.\n" . var_export($resapi, TRUE) . "\n" ) );
		return '';
	}

	/* get result, render start */
	$res0 = $resapi['ResultSet'][0]['Result'][0];

	/* generate embed HTML tags */
	$ret = '<div class="' . YJSHOPPING_CLASS_DIV;
	if( !empty($wgYJShoppingClass['div']) ) {
		$ret .= ' ' . $wgYJShoppingClass['div'];
	}
	$ret .= '" style="display: table;">';

	/* image and api credit */
	$ret .= '<div style="display: table-cell; vertical-align: top;"><a href="' . $res0['Url'] . '" target="_blank" rel="nofollow">';
	$ret .= '<img src="' . $res0['Image']['Medium'];
	if( !empty($wgYJShoppingClass['img']) ) {
		$ret .= '" class="' . $wgYJShoppingClass['img'];
	}
	$ret .= '" /></a>';

	/* https://developer.yahoo.co.jp/attribution/ */
	$ret .= '<br /><a href="' . YJSHOPPING_CREDIT_TARGET . '"><img src="' . YJSHOPPING_CREDIT_URL . '" border="0" width="' . YJSHOPPING_CREDIT_SIZE_X . '" height="' . YJSHOPPING_CREDIT_SIZE_Y . '" alt="' . YJSHOPPING_CREDIT_ALT . '" style="margin: 15px 15px 4px 5px;" /></a></div>';

	/* item name and link */
	$name = $res0['Name'];
	$maxlen = $wgYJShoppingMaxLen['name'] ? $wgYJShoppingMaxLen['name'] : YJSHOPPING_NAMEMAXLEN;
	$ret .= '<div style="display: table-cell; vertical-align: top;"><p';
	if( !empty($wgYJShoppingClass['name']) ) {
		$ret .= ' class="' . $wgYJShoppingClass['name'] . '"';
	}
	$ret .= '><a href="' . $res0['Url'] . '" target="_blank" rel="nofollow">';
	if( $maxlen < 0 || strlen($name) <= $maxlen ) {
		$ret .= htmlspecialchars($name);
	} else {
		$ret .= htmlspecialchars(mb_strcut($name, 0, $maxlen)) . '…';
	}
	$ret .= '</a></p>';

	/* store name */
	$text = $res0['Store']['Name'];
	$maxlen = $wgYJShoppingMaxLen['store'] ? $wgYJShoppingMaxLen['store'] : YJSHOPPING_STOREMAXLEN;
	$ret .= '<p';
	if( !empty($wgYJShoppingClass['store']) ) {
		$ret .= ' class="' . $wgYJShoppingClass['store'] . '"';
	}
	$ret .= '>';
	if( $maxlen < 0 || strlen($text) <= $maxlen ) {
		$ret .= htmlspecialchars($text);
	} else {
		$ret .= htmlspecialchars(mb_strcut($text, 0, $maxlen)) . '…';
	}
	$ret .= '</p>';

	/* plus links */
	if( !empty($wgYJShoppingPlus) ) {
		$ret .= '<ul';
		if( !empty($wgYJShoppingClass['plus']) ) {
			$ret .= ' class="' . $wgYJShoppingClass['plus'] . '"';
		}
		$ret .= '>';

		/* add Plus links (option) */
		foreach(array_keys($wgYJShoppingPlus) as $k) {
			$retplus = '';
			if( $k == 'amazon' && gettype($wgYJShoppingPlus[$k]) == 'string' ) {
				/* plus Amazon Associate (option) */
				$retplus = wfRenderYJShoppingPlusAmazon( $wgYJShoppingPlus[$k], $value );
			} elseif( $k == 'rakuten' && gettype($wgYJShoppingPlus[$k]['lsid']) == 'string' ) {
				/* plus Rakuten (LinkShare) (option) */
				$retplus = wfRenderYJShoppingPlusRakutenLS( $wgYJShoppingPlus[$k]['lsid'], $value );
			} elseif( $k == 'rakuten' && gettype($wgYJShoppingPlus[$k]) == 'string' ) {
				/* plus Rakuten (option) */
				$retplus = wfRenderYJShoppingPlusRakuten( $wgYJShoppingPlus[$k], $value );
			} elseif( !empty($wgYJShoppingPlus[$k]['sid']) && !empty($wgYJShoppingPlus[$k]['pid']) ) {
				/* plus ValueCommerce (option) */
				if( ($k == 'honto' || $k == '7net') && empty($querytype)) {
					/* honto, 7net options are ISBN/JAN only */
					continue;
				}
				$retplus = wfRenderYJShoppingPlusVC( $k, $wgYJShoppingPlus[$k]['sid'], $wgYJShoppingPlus[$k]['pid'], $value );
			} elseif( !empty($wgYJShoppingPlus[$k]['lsid']) && !empty($wgYJShoppingPlus[$k]['bids']) ) {
				/* plus LinkShare (option) */
				if( ($k == 'sofmap' || $k == 'sofmapused') && $querytype == YJSHOPPING_QUERYTYPE_ISBN) {
					/* sofmap, sofmapused options are keyword/JAN only */
					continue;
				}
				$retplus = wfRenderYJShoppingPlusLS( $k, $wgYJShoppingPlus[$k]['lsid'], $wgYJShoppingPlus[$k]['bids'], $value );
			}

			if( !empty($retplus) ) {
				$ret .= '<li ';
				if( !empty($wgYJShoppingClass[$k]) ) {
					$ret .= 'class="' . $wgYJShoppingClass[$k] . '"';
				}
				$ret .= '>' . $retplus . '</li>';
			}
		}

		/* plus kakakucom (option) */
		if( empty($querytype) && $wgYJShoppingPlus['kakakucom'] ) {
			$caption = wfMessage( 'yjshopping-plus-kakakucom' )->inContentLanguage()->escaped();
			$ret .= '<li class="' . $wgYJShoppingClass['kakakucom'] . '"><a href="http://kakaku.com/search_results/' . urlencode($value) . '/" title="' . wfMessage( 'yjshopping-plus-title', $value, $caption )->inContentLanguage()->escaped() . '" target="_blank" rel="nofollow">' . $caption . '</a></li>';
		}

		$ret .= '</ul>';
	}
	$ret .= "</div></div>\n";

	/* for debug */	YJSHOPPING_DEBUG && ( wfDebugLog( YJSHOPPING_HOOKID, "wfRenderYJShopping() complete: \n$ret\n" ) );

	return array( $ret, "markerType" => 'nowiki' );
//	return $ret;	/* MediaWikiを通した出力が中途半端にescapeされる */
//	return Html::rawElement( 'div', [ 'class' => YJSHOPPING_CLASS_DIV ], $ret );	/* 〃 */
#	return array( Html::rawElement( 'div', [ 'class' => YJSHOPPING_CLASS_DIV . ' ' . $wgYJShoppingClass['div'] ], $ret ), "markerType" => 'nowiki' );
}


/* for generate Value-Commerce link
 *
 */
function wfRenderYJShoppingPlusVC( $target, $sid, $pid, $keyword ) {
	/* incomplete parameters */
	if( empty($target) || empty($sid) || empty($pid) || empty($keyword) ) {
		return '';
	}

	$target = mb_strtolower($target);

	/* backward compatibility */
	if( strcmp($target, 'yj') === 0 ) {
		$target = 'yahoo';
	}

	/* undefined message */
	if( empty( wfMessage( 'yjshopping-plus-' . $target )->inContentLanguage() ) ) {
		/* for debug */	YJSHOPPING_DEBUG && ( wfDebugLog( YJSHOPPING_HOOKID, "wfRenderYJShoppingPlusVC(): yjshopping-plus-$target not defined.\n" ) );
		return '';
	}
	if( empty( wfMessage( 'yjshopping-plus-vc_url-' . $target )->inContentLanguage() ) ) {
		/* for debug */	YJSHOPPING_DEBUG && ( wfDebugLog( YJSHOPPING_HOOKID, "wfRenderYJShoppingPlusVC(): yjshopping-plus-vc_url-$target not defined.\n" ) );
		return '';
	}

	$caption = wfMessage( 'yjshopping-plus-' . $target )->inContentLanguage()->escaped();
	$vcurl = wfMessage( 'yjshopping-plus-vc_url-' . $target )->inContentLanguage()->rawParams( urlencode($keyword) )->plain();
	/* for debug */	YJSHOPPING_DEBUG && ( wfDebugLog( YJSHOPPING_HOOKID, "wfRenderYJShoppingPlusVC(): target, caption, vcurl:  $target, $caption, $vcurl\n" ) );

	return '<a href="https://ck.jp.ap.valuecommerce.com/servlet/referral?sid=' . $sid . '&pid=' . $pid . '&vc_url=' . urlencode($vcurl) . '" title="' . wfMessage( 'yjshopping-plus-title', $keyword, $caption )->inContentLanguage()->escaped() . '" target="_blank" rel="nofollow">' . $caption . '<img src="https://ad.jp.ap.valuecommerce.com/servlet/gifbanner?sid=' . $sid . '&pid=' . $pid . '" height="1" width="1" border="0" /></a>';
}


/* for generate LinkShare link
 *
 */
function wfRenderYJShoppingPlusLS( $target, $id, $bids, $keyword ) {
	/* incomplete parameters */
	if( empty($target) || empty($id) || empty($bids) || empty($keyword) ) {
		return '';
	}

	$target = mb_strtolower($target);

	/* backward compatibility */
	if( strcmp($target, 'yj') === 0 ) {
		$target = 'yahoo';
	}

	/* undefined message */
	if( empty( wfMessage( 'yjshopping-plus-' . $target )->inContentLanguage() ) ) {
		/* for debug */	YJSHOPPING_DEBUG && ( wfDebugLog( YJSHOPPING_HOOKID, "wfRenderYJShoppingPlusVC(): yjshopping-plus-$target not defined.\n" ) );
		return '';
	}
	if( empty( wfMessage( 'yjshopping-plus-ls_murl-' . $target )->inContentLanguage() ) ) {
		/* for debug */	YJSHOPPING_DEBUG && ( wfDebugLog( YJSHOPPING_HOOKID, "wfRenderYJShoppingPlusLS(): yjshopping-plus-ls_murl-$target not defined.\n" ) );
		return '';
	}

	$caption = wfMessage( 'yjshopping-plus-' . $target )->inContentLanguage()->escaped();
	$murl = wfMessage( 'yjshopping-plus-ls_murl-' . $target )->inContentLanguage()->rawParams( $id, urlencode($keyword) )->plain();
	/* for debug */	YJSHOPPING_DEBUG && ( wfDebugLog( YJSHOPPING_HOOKID, "wfRenderYJShoppingPlusLS(): id, target, caption, murl: $id, $target, $caption, $murl\n" ) );

	return '<a href="' . $murl . '" title="' . wfMessage( 'yjshopping-plus-title', $keyword, $caption )->inContentLanguage()->escaped() . '" target="_blank" rel="nofollow">' . $caption . '</a><img width="1" height="1" border="0" src="https://ad.linksynergy.com/fs-bin/show?id=U4SUyajTXJo&bids=' . $bids . '&type=10" />';
}


/* for generate Amazon Associate link
 *
 */
function wfRenderYJShoppingPlusAmazon( $atag, $keyword ) {
	/* incomplete parameters */
	if( empty($atag) || empty($keyword) ) {
		return '';
	}
	/* undefined message */
	if( empty( $caption = wfMessage( 'yjshopping-plus-amazon' )->inContentLanguage()->plain() ) ) {
		return '';
	}

	return '<a href="https://www.amazon.co.jp/gp/search?keywords=' . urlencode($keyword) . '&__mk_ja_JP=%E3%82%AB%E3%82%BF%E3%82%AB%E3%83%8A&tag=' . $atag . '" title="' . wfMessage( 'yjshopping-plus-title', $keyword, $caption )->inContentLanguage()->escaped() . '" target="_blank" rel="nofollow">' . $caption . '</a>';
}


/* for generate Rakuten Affiliate link
 *
 */
function wfRenderYJShoppingPlusRakuten( $aid, $keyword ) {
	/* incomplete parameters */
	if( empty($aid) || empty($keyword) ) {
		return '';
	}
	/* undefined message */
	if( empty( $caption = wfMessage( 'yjshopping-plus-rakuten' )->inContentLanguage()->plain() ) ) {
		return '';
	}

	return '<a href="https://hb.afl.rakuten.co.jp/hgc/' . urlencode($aid) . '/?pc=https%3A%2F%2Fsearch.rakuten.co.jp%2Fsearch%2Fmall%2F' . urlencode($keyword) . '%2F-%2Ff.1-p.1-s.1-sf.0-st.A-v.2%3Fx%3D0&scid=af_ich_link_urltxt" title="' . wfMessage( 'yjshopping-plus-title', $keyword, $caption )->inContentLanguage()->escaped() . '" target="_blank" rel="nofollow">' . $caption . '</a>';
}


/* for generate Rakuten LinkShare Affiliate link
 *
 */
function wfRenderYJShoppingPlusRakutenLS( $id, $keyword ) {
	/* incomplete parameters */
	if( empty($id) || empty($keyword) ) {
		return '';
	}
	/* undefined message */
	if( empty( $caption = wfMessage( 'yjshopping-plus-rakuten' )->inContentLanguage()->plain() ) ) {
		return '';
	}

	return '<a href="https://click.linksynergy.com/fs-bin/click?id=' . urlencode($id) . '&offerid=671770&RD_PARM1=https%253A%252F%252Fsearch.rakuten.co.jp%252Fsearch%252Fmall%252F' . urlencode($keyword) . '--hybrid--https%253A%252F%252Fsearch.rakuten.co.jp%252Fsearch%252Fmall%252F' . urlencode($keyword) . '&subid=&type=10&tmpid=11045" title="' . wfMessage( 'yjshopping-plus-title', $keyword, $caption )->inContentLanguage()->escaped() . '" target="_blank" rel="nofollow">' . $caption . '</a>';
}

?>
